package sun.security.krb5.internal;

import sun.security.krb5.KrbTgsReq;
import java.util.LinkedList;
import sun.security.krb5.Config;
import sun.security.krb5.Realm;
import java.io.IOException;
import sun.security.krb5.KrbException;
import sun.security.krb5.Credentials;
import sun.security.krb5.PrincipalName;

public class CredentialsUtil
{
    private static boolean DEBUG;
    
    public CredentialsUtil() {
    }
    
    public static Credentials acquireS4U2selfCreds(PrincipalName o, Credentials credentials) throws KrbException, IOException {
        if (!o.getRealmString().equals(credentials.getClient().getRealmString())) {
            throw new KrbException("Cross realm impersonation not supported");
        }
        if (!credentials.isForwardable()) {
            throw new KrbException("S4U2self needs a FORWARDABLE ticket");
        }
        KDCOptions with = KDCOptions.with(new int[] { 1 });
        PrincipalName client = credentials.getClient();
        PrincipalName client2 = credentials.getClient();
        Ticket[] array = null;
        PAData[] array2 = { null };
        array2[0] = new PAData(129, new PAForUserEnc(o, credentials.getSessionKey()).asn1Encode());
        Credentials serviceCreds = serviceCreds(with, credentials, client, client2, array, array2);
        if (!serviceCreds.getClient().equals(o)) {
            throw new KrbException("S4U2self request not honored by KDC");
        }
        if (!serviceCreds.isForwardable()) {
            throw new KrbException("S4U2self ticket must be FORWARDABLE");
        }
        return serviceCreds;
    }
    
    public static Credentials acquireS4U2proxyCreds(String name, Ticket ticket, PrincipalName o, Credentials credentials) throws KrbException, IOException {
        Credentials serviceCreds = serviceCreds(KDCOptions.with(new int[] { 14, 1 }), credentials, credentials.getClient(), new PrincipalName(name), new Ticket[] { ticket }, (PAData[])null);
        if (!serviceCreds.getClient().equals(o)) {
            throw new KrbException("S4U2proxy request not honored by KDC");
        }
        return serviceCreds;
    }
    
    public static Credentials acquireServiceCreds(String name, Credentials credentials) throws KrbException, IOException {
        return serviceCreds(new PrincipalName(name, 0), credentials);
    }
    
    private static Credentials getTGTforRealm(String cRealm, String anObject, Credentials credentials, boolean[] array) throws KrbException {
        String[] realmsList = Realm.getRealmsList(cRealm, anObject);
        Credentials credentials2 = null;
        array[0] = true;
        Credentials credentials3 = credentials;
        int i = 0;
        while (i < realmsList.length) {
            PrincipalName tgsService = PrincipalName.tgsService(anObject, realmsList[i]);
            if (CredentialsUtil.DEBUG) {
                System.out.println(">>> Credentials acquireServiceCreds: main loop: [" + i + "] tempService=" + tgsService);
            }
            Credentials credentials4;
            try {
                credentials4 = serviceCreds(tgsService, credentials3);
            }
            catch (Exception ex) {
                credentials4 = null;
            }
            if (credentials4 == null) {
                if (CredentialsUtil.DEBUG) {
                    System.out.println(">>> Credentials acquireServiceCreds: no tgt; searching thru capath");
                }
                credentials4 = null;
                for (int j = i + 1; credentials4 == null && j < realmsList.length; ++j) {
                    PrincipalName tgsService2 = PrincipalName.tgsService(realmsList[j], realmsList[i]);
                    if (CredentialsUtil.DEBUG) {
                        System.out.println(">>> Credentials acquireServiceCreds: inner loop: [" + j + "] tempService=" + tgsService2);
                    }
                    try {
                        credentials4 = serviceCreds(tgsService2, credentials3);
                    }
                    catch (Exception ex2) {
                        credentials4 = null;
                    }
                }
            }
            if (credentials4 == null) {
                if (CredentialsUtil.DEBUG) {
                    System.out.println(">>> Credentials acquireServiceCreds: no tgt; cannot get creds");
                    break;
                }
                break;
            }
            else {
                String instanceComponent = credentials4.getServer().getInstanceComponent();
                if (array[0] && !credentials4.checkDelegate()) {
                    if (CredentialsUtil.DEBUG) {
                        System.out.println(">>> Credentials acquireServiceCreds: global OK-AS-DELEGATE turned off at " + credentials4.getServer());
                    }
                    array[0] = false;
                }
                if (CredentialsUtil.DEBUG) {
                    System.out.println(">>> Credentials acquireServiceCreds: got tgt");
                }
                if (instanceComponent.equals(anObject)) {
                    credentials2 = credentials4;
                    break;
                }
                int n;
                for (n = i + 1; n < realmsList.length && !instanceComponent.equals(realmsList[n]); ++n) {}
                if (n < realmsList.length) {
                    i = n;
                    credentials3 = credentials4;
                    if (!CredentialsUtil.DEBUG) {
                        continue;
                    }
                    System.out.println(">>> Credentials acquireServiceCreds: continuing with main loop counter reset to " + i);
                }
                else {
                    break;
                }
            }
        }
        return credentials2;
    }
    
    private static Credentials serviceCreds(PrincipalName principalName, Credentials credentials) throws KrbException, IOException {
        return serviceCreds(new KDCOptions(), credentials, credentials.getClient(), principalName, null, (PAData[])null);
    }
    
    private static Credentials serviceCreds(KDCOptions kdcOptions, Credentials credentials, PrincipalName principalName, PrincipalName principalName2, Ticket[] array, PAData[] array2) throws KrbException, IOException {
        if (!Config.DISABLE_REFERRALS) {
            try {
                return serviceCredsReferrals(kdcOptions, credentials, principalName, principalName2, array, array2);
            }
            catch (KrbException ex) {}
        }
        return serviceCredsSingle(kdcOptions, credentials, principalName, credentials.getClientAlias(), principalName2, principalName2, array, array2);
    }
    
    private static Credentials serviceCredsReferrals(KDCOptions kdcOptions, Credentials creds, PrincipalName principalName, PrincipalName o, Ticket[] array, PAData[] array2) throws KrbException, IOException {
        kdcOptions = new KDCOptions(kdcOptions.toBooleanArray());
        kdcOptions.set(15, true);
        o = (o = o);
        Credentials serviceCredsSingle = null;
        int n = 0;
        LinkedList<String> list = new LinkedList();
        PrincipalName clientAlias = creds.getClientAlias();
        while (list.size() <= Config.MAX_REFERRALS) {
            ReferralsCache.ReferralCacheEntry value = ReferralsCache.get(principalName, o, o.getRealmString());
            String toRealm = null;
            if (value == null) {
                serviceCredsSingle = serviceCredsSingle(kdcOptions, creds, principalName, clientAlias, o, o, array, array2);
                PrincipalName server = serviceCredsSingle.getServer();
                if (!o.equals(server)) {
                    String[] nameStrings = server.getNameStrings();
                    if (nameStrings.length == 2 && nameStrings[0].equals("krbtgt") && !o.getRealmAsString().equals(nameStrings[1])) {
                        ReferralsCache.put(principalName, o, server.getRealmString(), nameStrings[1], serviceCredsSingle);
                        toRealm = nameStrings[1];
                        n = 1;
                        creds = serviceCredsSingle;
                    }
                }
            }
            else {
                toRealm = value.getToRealm();
                creds = value.getCreds();
                n = 1;
            }
            if (n != 0) {
                if (list.contains(toRealm)) {
                    return null;
                }
                o = new PrincipalName(o.getNameString(), o.getNameType(), toRealm);
                list.add(toRealm);
                n = 0;
            }
            else {
                break;
            }
        }
        return serviceCredsSingle;
    }
    
    private static Credentials serviceCredsSingle(KDCOptions kdcOptions, Credentials credentials, PrincipalName client, PrincipalName principalName, PrincipalName principalName2, PrincipalName principalName3, Ticket[] array, PAData[] array2) throws KrbException, IOException {
        boolean[] array3 = { true };
        String s = credentials.getServer().getNameStrings()[1];
        String realmString = principalName2.getRealmString();
        if (!realmString.equals(s)) {
            if (CredentialsUtil.DEBUG) {
                System.out.println(">>> serviceCredsSingle: cross-realm authentication");
                System.out.println(">>> serviceCredsSingle: obtaining credentials from " + s + " to " + realmString);
            }
            Credentials tgTforRealm = getTGTforRealm(s, realmString, credentials, array3);
            if (tgTforRealm == null) {
                throw new KrbApErrException(63, "No service creds");
            }
            if (CredentialsUtil.DEBUG) {
                System.out.println(">>> Cross-realm TGT Credentials serviceCredsSingle: ");
                Credentials.printDebug(tgTforRealm);
            }
            credentials = tgTforRealm;
            client = credentials.getClient();
        }
        else if (CredentialsUtil.DEBUG) {
            System.out.println(">>> Credentials serviceCredsSingle: same realm");
        }
        Credentials sendAndGetCreds = new KrbTgsReq(kdcOptions, credentials, client, principalName, principalName2, principalName3, array, array2).sendAndGetCreds();
        if (sendAndGetCreds != null) {
            if (CredentialsUtil.DEBUG) {
                System.out.println(">>> TGS credentials serviceCredsSingle:");
                Credentials.printDebug(sendAndGetCreds);
            }
            if (!array3[0]) {
                sendAndGetCreds.resetDelegate();
            }
        }
        return sendAndGetCreds;
    }
    
    static {
        CredentialsUtil.DEBUG = Krb5.DEBUG;
    }
}
